@@ -323,7 +323,7 @@ knumber_base *knumber_fraction::bitwise_shift(knumber_base *rhs) {
 	Q_UNUSED(rhs);
 	delete this;
 	// NOTE: we don't support bitwise operations with non-integer operands
-	return new knumber_integer(0);
+	return new knumber_error(knumber_error::ERROR_UNDEFINED);
 }
 
 //------------------------------------------------------------------------------
@@ -572,6 +572,24 @@ knumber_base *knumber_fraction::sin() {
 	return f->sin();
 }
 
+//------------------------------------------------------------------------------
+// Name:
+//------------------------------------------------------------------------------
+knumber_base *knumber_fraction::floor() {
+	knumber_float *f = new knumber_float(this);
+	delete this;
+	return f->floor();
+}
+
+//------------------------------------------------------------------------------
+// Name:
+//------------------------------------------------------------------------------
+knumber_base *knumber_fraction::ceil() {
+	knumber_float *f = new knumber_float(this);
+	delete this;
+	return f->ceil();
+}
+
 //------------------------------------------------------------------------------
 // Name:
 //------------------------------------------------------------------------------
@@ -582,6 +600,16 @@ knumber_base *knumber_fraction::cos() {
 	return f->cos();
 }
 
+//------------------------------------------------------------------------------
+// Name:
+//------------------------------------------------------------------------------
+knumber_base *knumber_fraction::tgamma() {
+
+	knumber_float *f = new knumber_float(this);
+	delete this;
+	return f->tgamma();
+}
+
 //------------------------------------------------------------------------------
 // Name:
 //------------------------------------------------------------------------------
